#include<string.h>
#include"argon.h"

static double mod(t_coord r)
{
	return sqrt(r.x*r.x+r.y*r.y+r.z*r.z);
}

double dist(t_coord r1, t_coord r2)
{
	t_coord r={r1.x-r2.x, r1.y-r2.y, r1.z-r2.z, 0};

	return mod(r);
}

t_coord scale(t_coord r, double s)
{
	t_coord res=r;

	res.x*=s;
	res.y*=s;
	res.z*=s;

	return res;
}

static double wall_force(t_parameters *pars; t_coord F_S[pars->N], t_state *state, t_parameters *pars)
{
	int N=pars->N;

	double res=0;

	for(int i=0; i<N; i++) {
		double mod_r=mod(state->r[i]);
		
		if(mod_r>=pars->L) {
			res+=0.5*pars->f*(mod_r-pars->L)*(mod_r-pars->L);

			F_S[i].x=pars->f*(pars->L-mod_r)/mod_r*state->r[i].x;
			F_S[i].y=pars->f*(pars->L-mod_r)/mod_r*state->r[i].y;
			F_S[i].z=pars->f*(pars->L-mod_r)/mod_r*state->r[i].z;

			/* This trick causes the wall to absorb excessive kinetic energy.*/
			if(mod_r>=(pars->L+pars->Lt)) {
				state->p[i].x=0;
				state->p[i].y=0;
				state->p[i].z=0;
			}
		} else {
			F_S[i].x=0.0;
			F_S[i].y=0.0;
			F_S[i].z=0.0;
		}
	}

	return res;
}

static double pressure(t_parameters *pars; t_coord F_S[pars->N], t_parameters *pars)
{
	int N=pars->N;
	double res=0;


	for(int i=0; i<N; i++) {
		res+=mod(F_S[i]);
	}

	res*=0.25/pars->L/pars->L*M_1_PI;

	return res;
}

static double vdv_force(t_parameters *pars; t_coord F_P[pars->N], t_state *state, t_parameters *pars)
{
	int N=pars->N;

	double res=0;

	for(int i=0; i<N; i++) {
		F_P[i].x=0.0;
		F_P[i].y=0.0;
		F_P[i].z=0.0;
	}

	for(int i=0; i<N; i++) {
		for(int j=i+1; j<N; j++) {
			t_coord r_ij;

			r_ij.x=state->r[i].x-state->r[j].x;
			r_ij.y=state->r[i].y-state->r[j].y;
			r_ij.z=state->r[i].z-state->r[j].z;

			double mod_r_ij=mod(r_ij);


			double pow6=pow(pars->R/mod_r_ij, 6);
			double pow12=pow6*pow6;

			res+=pars->e*(pow12-2*pow6);

			t_coord F_ij;

			double qq=12*pars->e*(pow12-pow6)/(mod_r_ij*mod_r_ij);

			
			F_ij.x=qq*r_ij.x;
			F_ij.y=qq*r_ij.y;
			F_ij.z=qq*r_ij.z;

			F_P[i].x+=F_ij.x;
			F_P[i].y+=F_ij.y;
			F_P[i].z+=F_ij.z;

			F_P[j].x-=F_ij.x;
			F_P[j].y-=F_ij.y;
			F_P[j].z-=F_ij.z;

		}
	}

	return res;
}


void compute_forces(t_state *state, t_parameters *pars)
{
	int N=pars->N;

	t_coord F_S[N];
	double V_S=wall_force(F_S, state, pars);

	double P=pressure(F_S, pars);

	t_coord F_P[N];
	double V_P=vdv_force(F_P, state, pars);

	for(int i=0; i<N; i++) {
		state->F[i].x=F_S[i].x+F_P[i].x;
		state->F[i].y=F_S[i].y+F_P[i].y;
		state->F[i].z=F_S[i].z+F_P[i].z;
	}

	state->V=V_S+V_P;

	state->P=P;
}


static double kinetic_energy(t_state *state, t_parameters *pars)
{
	int N=pars->N;

	double res=0;

	for(int i=0; i<N; i++) {
		double mod_r=mod(state->p[i]);
		
		res+=mod_r*mod_r;
	}

	return res*0.5/pars->m;
}

void one_step(t_state *state, t_parameters *pars)
{
	int N=pars->N;

	const t_coord zero={0,0,0,0};

	for(int i=0; i<N; i++) {
		if(!state->locked[i]) {

			/* dF is supplied by the user through a haptic interface.*/

			state->p[i].x=state->p[i].x+.5*(state->F[i].x+state->dF[i].x)*pars->tau;
			state->p[i].y=state->p[i].y+.5*(state->F[i].y+state->dF[i].y)*pars->tau;
			state->p[i].z=state->p[i].z+.5*(state->F[i].z+state->dF[i].z)*pars->tau;

			state->p[i]=scale(state->p[i], 1-state->damping[i]);

			state->damping[i]*=0.75;
		} else {
			state->p[i]=zero;
		}
	}

	for(int i=0; i<N; i++) {
		if(!state->locked[i]) {
			state->r[i].x=state->r[i].x+state->p[i].x*pars->tau/pars->m;
			state->r[i].y=state->r[i].y+state->p[i].y*pars->tau/pars->m;
			state->r[i].z=state->r[i].z+state->p[i].z*pars->tau/pars->m;
		}
	}

	compute_forces(state, pars);

	for(int i=0; i<N; i++) {
		state->p[i].x=state->p[i].x+.5*(state->F[i].x+state->dF[i].x)*pars->tau;
		state->p[i].y=state->p[i].y+.5*(state->F[i].y+state->dF[i].y)*pars->tau;
		state->p[i].z=state->p[i].z+.5*(state->F[i].z+state->dF[i].z)*pars->tau;
	}

	state->step++;
	state->t+=pars->tau;

	double E_kin=kinetic_energy(state, pars);

	state->T=2.0/3.0/N/K_BOLTZMANN*E_kin;
	state->H=E_kin+state->V;
}
